(text "{% extends \"root.html\" %} {% block head %}")
(title
    (text "My services - {{ config.name }}"))

(text "{% endblock %} {% block body %} {{ macros::nav() }}")
(main
    ("class" "flex flex-col gap-2")
    (text "{% if user -%}")
    (div
        ("class" "pillmenu")
        (a ("href" "/services") ("class" "active") (str (text "littleweb:label.services")))
        (a ("href" "/domains") (str (text "littleweb:label.domains"))))

    (div
        ("class" "card-nest")
        (div
            ("class" "card small flex flex-col gap-2")
            (div
                ("class" "flex w-full gap-2 justify-between")
                (b
                    (text "{{ service.name }}"))

                (button
                    ("class" "small lowered")
                    ("title" "Help")
                    ("onclick" "document.getElementById('site_help').classList.toggle('hidden')")
                    (icon (text "circle-question-mark"))))

            (div
                ("class" "card w-full lowered flex flex-col gap-2 hidden no_p_margin")
                ("id" "site_help")
                (p (text "Your site should include an \"index.html\" file in order to show content on its homepage."))
                (p (text "In the HTML editor, you can type `!` and use the provided suggestion to get an HTML boilerplate."))
                (p (text "After you've created a page, you can click \"Copy ID\" and go to manage a domain you own. On the domain management page, click \"Add\" and paste the ID you copied into the value field."))))

        (div
            ("class" "flex gap-2 flex-wrap card")
            (text "{% if file and file.children|length == 0 -%}")
            (button
                ("onclick" "update_content()")
                (icon (text "check"))
                (str (text "general:action.save")))
            (text "{%- endif %}")

            (button
                ("class" "lowered")
                ("onclick" "update_name()")
                (icon (text "pencil"))
                (str (text "littleweb:action.edit_site_name")))

            (button
                ("class" "lowered")
                ("onclick" "trigger('atto::copy_text', ['{{ service.id }}'])")
                (icon (text "copy"))
                (str (text "general:action.copy_id")))

            (button
                ("class" "red lowered")
                ("onclick" "delete_service()")
                (icon (text "trash"))
                (str (text "general:action.delete")))))
    (text "{%- endif %}")
    (div
        ("class" "card-nest w-full")
        (div
            ("class" "card small flex items-center justify-between gap-2")
            (div
                ("class" "flex items-center gap-2")
                (icon (text "folder-open"))
                (span (text "{% if path -%} {{ path }} {%- else -%} / {%- endif %}")))

            (div
                ("class" "flex items-center gap-2")
                (button
                    ("class" "lowered small")
                    ("onclick" "go_up()")
                    (icon (text "arrow-up")))

                (text "{% if not file or file.content|length == 0 -%}")
                (button
                    ("class" "lowered small")
                    ("onclick" "create_file()")
                    (icon (text "plus"))
                    (str (text "communities:action.create")))
                (text "{%- endif %}")))
        (div
            ("class" "card flex flex-col gap-2")
            (text "{% if not file or file.children|length > 0 -%}")
            ; directory browser
            (div
                ("class" "w-full")
                ("style" "max-width: 100%; overflow: auto; min-height: 512px")
                (table
                    ("class" "w-full")
                    (thead
                        (tr
                            (th (text "Name"))
                            (th (text "Type"))
                            (th (text "Children"))
                            (th (text "Actions"))))

                    (tbody
                        (text "{% for item in files %}")
                        (tr
                            (td
                                ("class" "flex gap-2 items-center")
                                (text "{% if item.children|length > 0 -%}")
                                (icon (text "folder"))
                                (text "{% else %}")
                                (icon (text "file"))
                                (text "{%- endif %}")

                                (a
                                    ("href" "?path={{ path }}/{{ item.name }}")
                                    ("data-turbo" "false")
                                    (text "{{ item.name }}")))
                            (td (text "{{ item.mime }}"))
                            (td (text "{{ item.children|length }}"))
                            (td
                                ("style" "overflow: auto")
                                (div
                                    ("class" "dropdown")
                                    (button
                                        ("class" "camo small")
                                        ("onclick" "trigger('atto::hooks::dropdown', [event])")
                                        ("exclude" "dropdown")
                                        (icon (text "ellipsis")))
                                    (div
                                        ("class" "inner")
                                        (button
                                            ("onclick" "rename_file('{{ item.id }}')")
                                            (icon (text "pencil"))
                                            (str (text "littleweb:action.rename")))

                                        (button
                                            ("class" "red")
                                            ("onclick" "remove_file('{{ item.id }}')")
                                            (icon (text "trash"))
                                            (str (text "general:action.delete")))))))
                        (text "{% endfor %}"))))
            (text "{% else %}")
            ; file editor
            (div ("id" "editor_container") ("class" "w-full") ("style" "height: 600px"))
            (text "{%- endif %}"))))

(script ("id" "all_service_files") ("type" "application/json") (text "{{ service.files|json_encode()|remove_script_tags|safe }}"))
(script ("id" "service_files") ("type" "application/json") (text "{{ files|json_encode()|remove_script_tags|safe }}"))
(script ("id" "id_path") ("type" "application/json") (text "{{ id_path|json_encode()|remove_script_tags|safe }}"))

(script
    (text "globalThis.SERVICE_FILES = JSON.parse(document.getElementById(\"service_files\").innerText);
    globalThis.EXTENSION_MIMES = {
        \"html\": \"text/html\",
        \"js\": \"text/javascript\",
        \"css\": \"text/css\",
        \"json\": \"application/json\",
        \"txt\": \"text/plain\",
    }

    globalThis.MIME_MODES = {
        \"Html\": \"html\",
        \"Js\": \"javascript\",
        \"Css\": \"css\",
        \"Json\": \"json\",
        \"Plain\": \"txt\",
    }

    function go_up() {
        const x = JSON.parse(document.getElementById(\"id_path\").innerText);
        const y = JSON.parse(document.getElementById(\"all_service_files\").innerText);

        x.pop();
        let path = \"\";

        for (id of x) {
            path += `/${y.find((x) => x.id == id).name}`;
        }

        window.location.href = `?path=${path}`;
    }

    async function update_name() {
        await trigger(\"atto::debounce\", [\"services::update_name\"]);

        const name = await trigger(\"atto::prompt\", [\"New name:\"]);

        if (!name) {
            return;
        }

        fetch(\"/api/v1/services/{{ service.id }}/name\", {
            method: \"POST\",
            headers: {
                \"Content-Type\": \"application/json\",
            },
            body: JSON.stringify({
                name: e.target.name.value,
            }),
        })
            .then((res) => res.json())
            .then((res) => {
                trigger(\"atto::toast\", [
                    res.ok ? \"success\" : \"error\",
                    res.message,
                ]);
            });
    }

    async function delete_service() {
        await trigger(\"atto::debounce\", [\"services::delete\"]);

        if (
            !(await trigger(\"atto::confirm\", [
                \"Are you sure you would like to do this?\",
            ]))
        ) {
            return;
        }

        fetch(\"/api/v1/services/{{ service.id }}\", {
            method: \"DELETE\",
        })
            .then((res) => res.json())
            .then((res) => {
                trigger(\"atto::toast\", [
                    res.ok ? \"success\" : \"error\",
                    res.message,
                ]);
            });
    }

    async function update_content() {
        await trigger(\"atto::debounce\", [\"services::update_content\"]);
        const content = globalThis.editor.getValue();
        fetch(\"/api/v1/services/{{ service.id }}/content\", {
            method: \"POST\",
            headers: {
                \"Content-Type\": \"application/json\",
            },
            body: JSON.stringify({
                content,
                id_path: JSON.parse(document.getElementById(\"id_path\").innerText),
            }),
        })
            .then((res) => res.json())
            .then((res) => {
                trigger(\"atto::toast\", [
                    res.ok ? \"success\" : \"error\",
                    res.message,
                ]);
            });
    }

    async function update_files() {
        await trigger(\"atto::debounce\", [\"services::update_files\"]);
        fetch(\"/api/v1/services/{{ service.id }}/files\", {
            method: \"POST\",
            headers: {
                \"Content-Type\": \"application/json\",
            },
            body: JSON.stringify({
                files: SERVICE_FILES,
                id_path: JSON.parse(document.getElementById(\"id_path\").innerText),
            }),
        })
            .then((res) => res.json())
            .then((res) => {
                trigger(\"atto::toast\", [
                    res.ok ? \"success\" : \"error\",
                    res.message,
                ]);
            });
    }

    async function create_file() {
        await trigger(\"atto::debounce\", [\"services::create_file\"]);

        let name = await trigger(\"atto::prompt\", [\"Name:\"]);

        if (!name) {
            return;
        }

        const s = name.split(\".\");
        SERVICE_FILES.push({
            id: window.crypto.randomUUID(),
            name,
            mime: EXTENSION_MIMES[s[s.length - 1]] || EXTENSION_MIMES[\"txt\"],
            children: [],
            content: \"\",
        });

        await update_files();

        setTimeout(() => {
            window.location.reload();
        }, 150);
    }

    async function rename_file(id) {
        await trigger(\"atto::debounce\", [\"services::rename_file\"]);

        let name = await trigger(\"atto::prompt\", [\"New name:\"]);

        if (!name) {
            return;
        }

        const file_ref = SERVICE_FILES.find((x) => x.id === id);
        file_ref.name = name;

        const s = name.split(\".\");
        file_ref.mime = EXTENSION_MIMES[s[s.length - 1]] || EXTENSION_MIMES[\"txt\"];

        await update_files();
    }

    async function remove_file(id) {
        await trigger(\"atto::debounce\", [\"services::remove_file\"]);

        if (
            !(await trigger(\"atto::confirm\", [
                \"Are you sure you would like to do this?\",
            ]))
        ) {
            return;
        }

        let i = 0;
        SERVICE_FILES.find((x) => {
            i += 1;
            return x.id === id;
        });

        SERVICE_FILES.splice(i - 1, 1);
        await update_files();
    }"))

(text "{% if file and file.mime != 'Plain' -%}")
(script ("src" "https://unpkg.com/monaco-editor@0.52.2/min/vs/loader.js"))
(script ("src" "https://unpkg.com/emmet-monaco-es/dist/emmet-monaco.min.js"))
(script ("id" "file_content") ("type" "text/plain") (text "{{ file.content|remove_script_tags|safe }}"))
(script
    (text "require.config({ paths: { vs: \"https://unpkg.com/monaco-editor@0.52.2/min/vs\" } });

	require([\"vs/editor/editor.main\"], () => {
	    const shadow = document.getElementById(\"editor_container\").attachShadow({
			mode: \"closed\",
		});

		const inner = document.createElement(\"div\");
		inner.style.width = window.getComputedStyle(document.getElementById(\"editor_container\")).width;
		inner.style.height = window.getComputedStyle(document.getElementById(\"editor_container\")).height;
		shadow.appendChild(inner);

		const style = document.createElement(\"style\");
		style.innerText = '@import \"https://unpkg.com/monaco-editor@0.52.2/min/vs/editor/editor.main.css\";';
		shadow.appendChild(style);

		emmetMonaco.emmetHTML();
		emmetMonaco.emmetCSS();

		globalThis.editor = monaco.editor.create(inner, {
			value: document.getElementById(\"file_content\").innerText.replaceAll(\"&lt;/script&gt;\", \"</script\" + \">\"),
			language: MIME_MODES[\"{{ file.mime }}\"],
			theme: \"vs-dark\",
			suggest: {
                snippetsPreventQuickSuggestions: false,
            },
		});
	});"))
(text "{%- endif %}")
(text "{% endblock %}")
